const chalk = require("chalk");
const OSS = require('ali-oss');
const fs = require("fs");
const path = require("path");
const inquirer = require('inquirer');
const ora = require("ora");
const axios = require("axios");
const { cdnDomain, ajaxBaseUrl, cliName, cosConfig } = require("./config");

// 引入腾讯cos模块
const COS = require('cos-nodejs-sdk-v5');
var cos = new COS(cosConfig);

// 请求loading
const waitLoading = (fn, message) => async (...args) => {
  const spinner = ora(message);
  spinner.start()
  const result = await fn(...args)
  spinner.succeed()
  return result;
}
/**
 * http://gitlab.redcat.im/hujili/zyt-backstage.git

 * 
 * projectName hujili/zyt-backstage
 * currentBranch daily
 * publishDirName build
 */
module.exports = async (userInfo, projectName, currentBranch, publishDirName) => {
  try {
    if (!publishDirName) throw `请指定发布文件夹;如 ${cliName} cdn build`;

    const publishDirPath = path.join(process.cwd(), publishDirName);

    if (!fs.existsSync(publishDirPath)) throw `路径 ${publishDirPath} 不存在`;

    if (!fs.statSync(publishDirPath).isDirectory()) throw `路径 ${publishDirPath} 不是文件夹`;

    let isVersion = false;
    let nextVersionData = {
      version: "0.0.1"
    };
    // 选择按版本发布还是直接发布
    const publishMot = await inquirer.prompt([
      {
        type: 'list',
        message: '请选择发布模式',
        name: 'publishType',
        choices: [
          "直接发布",
          "按版本发布"
        ]
      }
    ]);
    let inputRemark = {}

    if (publishMot.publishType === "按版本发布") {
      // 输入备注信息
      inputRemark = await inquirer.prompt([
        {
          type: 'input',
          message: '请输入发布备注',
          name: 'remark',
          validate: val => { if (!val) return "备注不能为空"; return true }
        }
      ]);
      isVersion = true;
      const currentVersionData = await waitLoading(findAppVersion, "正在查看当前版本")(projectName, currentBranch);
      if (!currentVersionData.status) throw currentVersionData.message;

      if (currentVersionData.entry) {
        nextVersionData.id = currentVersionData.entry.id;
        const currentVersion = currentVersionData.entry.current_version;
        console.log(chalk.green(`当前版本 ${currentVersion}`));
        const versionList = currentVersion.split(".");
        let big_v = Number(versionList[0]);
        let mid_v = Number(versionList[1]);
        let small_v = Number(versionList[2]);
        small_v++
        if (small_v === 10) {
          mid_v++
          small_v = 0
        }
        if (mid_v === 10) {
          big_v++
          mid_v = 0
        }
        nextVersionData.version = `${big_v}.${mid_v}.${small_v}`;
        console.log(chalk.green(`发布版本 ${nextVersionData.version}`));
      } else {
        console.log("你还没发布过此应用，将为你创建0.0.1版本");
      }
    }
    console.log(chalk.yellow(`发布地址-> ${cdnDomain}`));
    console.log(chalk.yellow("开始发布"));
    await uploadDirToCdn({
      isVersion,
      nextVersion: nextVersionData.version,
      publishDirPath,
      projectName,
      currentBranch,
      publishDirName
    });
    if (isVersion) {
      if (nextVersionData.version === "0.0.1") {
        const newAppVersionReturn = await publishNewAppVersion(projectName, currentBranch);
        if (!newAppVersionReturn.status) throw newAppVersionReturn.message;
        nextVersionData.id = newAppVersionReturn.entry.insertId;
      } else {
        const updateAppVersionReturn = await updateAppVersion(nextVersionData.id, nextVersionData.version);
        if (!updateAppVersionReturn.status) throw updateAppVersionReturn.message;
      }
    }
    // 写发布记录
    const addPublishLogResult = await addPublishLog({
      publisher: userInfo.name,
      env: currentBranch,
      app_name: projectName,
      current_version: isVersion ? nextVersionData.version : null,
      cdn_publish_app_id: isVersion ? nextVersionData.id : null,
      remark: inputRemark.remark
    });
    if (!addPublishLogResult.status) throw addPublishLogResult.message;
    console.log(chalk.green("发布成功！"))
  } catch (err) {
    console.log(chalk.red(err))
  }
}
/**
 * 添加发布记录 
 */
async function addPublishLog ({ publisher, env, app_name, current_version, cdn_publish_app_id, remark }) {
  return axios({
    url: `${ajaxBaseUrl}/cdnPublishLog/add`,
    method: "post",
    data: {
      publisher,
      env,
      app_name,
      current_version,
      cdn_publish_app_id,
      remark
    }
  }).then(res => res.data);
}

/**
 * 查找app版本号
 */
async function findAppVersion (projectName, currentBranch) {
  return axios({
    url: `${ajaxBaseUrl}/cdnPublishApp/currentVersion`,
    method: "get",
    params: {
      app_name: projectName,
      env: currentBranch,
    }
  }).then(res => res.data);
}

/**
 * 发布新版本项目
 */
async function publishNewAppVersion (projectName, currentBranch) {
  return axios({
    url: `${ajaxBaseUrl}/cdnPublishApp/insertNew`,
    method: "post",
    data: {
      app_name: projectName,
      env: currentBranch,
    }
  }).then(res => res.data);
}

/**
 * 更新项目版本
 */
async function updateAppVersion (id, current_version) {
  return axios({
    url: `${ajaxBaseUrl}/cdnPublishApp/updateVersion`,
    method: "post",
    data: {
      id,
      current_version
    }
  }).then(res => res.data);
}

/**
 * 上传文件夹到oss cdn
 * publishDirPath 发布文件夹本地路径
 * projectName 项目名 hujili/zyt-backstage
 * currentBranch 当前分支 daily
 * publishDirName 发布目录名 build
 * 
 * https://cdn.hrwork.com/hujili/zyt-backstage/publish/ build /asset-manifest.json
 * https://cdn.hrwork.com/hrwork/hrwork-react-antd/graym/ 2.8.1 /dist/Account.async.js
 * https://pages.qejs.tencent-cloud.com//hknet/zhibo-topbusiness/index.html
 */
async function uploadDirToCdn ({ publishDirPath, projectName, currentBranch, publishDirName, isVersion, nextVersion }) {
  let files = fs.readdirSync(publishDirPath)
  for (const filename of files) {
    let filedir = path.join(publishDirPath, filename).replace(/\\/g, "/");;
    let stats = fs.statSync(filedir)
    let isFile = stats.isFile();
    let isDir = stats.isDirectory();
    if (isFile) {
      let filePathName = filedir.split(`${publishDirName}/`)[1];
      let ossPublishPath
      if (isVersion) {
        ossPublishPath = `${projectName}/${currentBranch}/${nextVersion}/${publishDirName}/${filePathName}`
      } else {
        ossPublishPath = `${projectName}/${currentBranch}/${publishDirName}/${filePathName}`
      }
      // 阿里云推送
      // let result = await ossClient.put(ossPublishPath, path.join(filedir))
      // 腾讯云推送
      cos.putObject({
        ...cosConfig, 
        Key: ossPublishPath,    /** 必须 对象键（Object 的名称），对象在存储桶中的唯一标识 */
        StorageClass: 'STANDARD',/** 设置对象的存储类型，枚举值：STANDARD、STANDARD_IA、ARCHIVE */
        Body: fs.createReadStream(filedir), /** filePath本地文件路径 上传文件的内容，可以为 FileStream、字符串、Buffer	Stream/Buffer/String */
        onProgress: function(progressData) {
            // console.log(JSON.stringify(progressData));/** 进度的回调函数，进度回调响应对象（progressData）属性如下 */
        }
      }, function(err, data) {
        console.log("up tencent-cos -> " + (!err? chalk.green(cdnDomain + "/" + ossPublishPath):  chalk.red(cdnDomain + "/" + ossPublishPath)))
      });
    }
    if (isDir) {
      await uploadDirToCdn({
        publishDirPath: filedir,
        projectName,
        currentBranch,
        publishDirName,
        isVersion, nextVersion
      });
    }
  }
}