const chalk = require("chalk");
const OSS = require('ali-oss');
const fs = require("fs");
const path = require("path");
const inquirer = require('inquirer');
const ora = require("ora");
const axios = require("axios");
const { ajaxBaseUrl, cosConfig, cdnDomain} = require("./config");
// 引入腾讯cos模块
const COS = require('cos-nodejs-sdk-v5');
var cos = new COS(cosConfig);

module.exports = async (userInfo, projectName, currentBranch, publishFileName) => {
  try {
    if (!publishFileName) {
      throw "请指定发布文件; 如 zhibo-p page build/index.html";
    }
    // 校验文件是否存在
    const publishFilePath = path.join(process.cwd(), publishFileName);
    if (!fs.existsSync(publishFilePath)) {
      throw `文件 ${publishFilePath} 不存在`;
    }
    // 校验是不是文件
    if (!fs.statSync(publishFilePath).isFile()) {
      throw `路径 ${publishFilePath} 不是文件`;
    }
    // 输入备注信息
    const inputRemark = await inquirer.prompt([
      {
        type: 'input',
        message: '请输入发布备注',
        name: 'remark',
        validate: val => { if (!val) return "备注不能为空"; return true }
      }
    ]);
    console.log(chalk.yellow("开始发布"));
    // 发布的文件名
    const file_name = path.basename(publishFileName); // index.html
    // 配置应用，返回id
    const pagePublishAppRes = await configApp({
      app_name: projectName,
      env: currentBranch,
      file_name
    });
    console.log(pagePublishAppRes, "pagePublishAppRes")
    if (!pagePublishAppRes.status) throw pagePublishAppRes.message;
    // 添加发布记录
    const addPagePublishLogRes = await addPagePublishLog({
      page_publish_app_id: pagePublishAppRes.entry.id,
      publisher: userInfo.name,
      remark: inputRemark.remark,
    });
    console.log(addPagePublishLogRes, "addPagePublishLogRes")
    if (!addPagePublishLogRes.status) throw addPagePublishLogRes.message;
    const pagePublishLogId = addPagePublishLogRes.entry;
    // 保存发布文件
    const content = fs.readFileSync(publishFilePath, 'utf-8');
    const savePublishFileRes = await savePublishFile({
      page_publish_log_id: pagePublishLogId,
      app_name: projectName,
      content,
      env: currentBranch
    });
    console.log(savePublishFileRes, "savePublishFileRes--savePublishFileRes")
    if (!savePublishFileRes.status) throw savePublishFileRes.message;
    // 1.发到阿里云oss
    // const ossClientConfigRes = await getOssClient({ env: currentBranch });
    // console.log(ossClientConfigRes, "ossClientConfigRes")
    // if (!ossClientConfigRes.status) throw ossClientConfigRes.message;

    // const { region, bucket, domain, accessKeyId, accessKeySecret } = ossClientConfigRes.entry;
    // const ossClient = new OSS({
    //   region,
    //   accessKeyId,
    //   accessKeySecret,
    //   bucket
    // });
    // // 页面发布，是定向发布到哪个bucket
    // const resultOss = await ossClient.put(`${pagePublishAppRes.entry.oss_app_name}/${file_name}`, publishFilePath);
    // console.log(chalk.yellow("发布成功"));
    // console.log("发布地址 " + chalk.green(domain + "/" + resultOss.name));


    // 2.发布到腾讯云
    let ossPublishPath = `${pagePublishAppRes.entry.oss_app_name}/${file_name}`
    console.log(publishFilePath,'publishFilePath')
    cos.putObject({
      ...cosConfig, 
      Key: ossPublishPath,    /** 必须 对象键（Object 的名称），对象在存储桶中的唯一标识 */
      StorageClass: 'STANDARD',/** 设置对象的存储类型，枚举值：STANDARD、STANDARD_IA、ARCHIVE */
      Body:  fs.createReadStream(publishFilePath), /** 上传文件的内容，可以为 FileStream、字符串、Buffer	Stream/Buffer/String */
      onProgress: function(progressData) {
          // console.log(JSON.stringify(progressData));/** 进度的回调函数，进度回调响应对象（progressData）属性如下 */
      }
    }, function(err, data) {
      console.log("up tencent-cos -> " + (!err? chalk.green(cdnDomain + "/" + ossPublishPath):  chalk.red(cdnDomain + "/" + ossPublishPath)))
      console.log("发布地址 " + chalk.green(cdnDomain + "/" + ossPublishPath));
    });
  } catch (err) {
    console.log(chalk.red(err))
  }
}

// 配置应用，返回id
async function configApp ({ app_name, env, file_name }) {
  return axios({
    url: `${ajaxBaseUrl}/pagePublishApp/config`,
    method: "post",
    data: {
      app_name,
      env,
      file_name
    }
  }).then(res => res.data);
}

// 添加发布记录
async function addPagePublishLog ({ page_publish_app_id, publisher, remark }) {
  return axios({
    url: `${ajaxBaseUrl}/pagePublishLog/add`,
    method: "post",
    data: {
      page_publish_app_id,
      publisher,
      remark,
    }
  }).then(res => res.data);
}

// 保存发布文件
async function savePublishFile ({ page_publish_log_id, content, app_name, env }) {
  return axios({
    url: `${ajaxBaseUrl}/pagePublishLog/saveFile`,
    method: "post",
    data: {
      page_publish_log_id,
      content,
      app_name,
      env
    }
  }).then(res => res.data);
}

// 配置oss client
async function getOssClient ({ env }) {
  return axios({
    url: `${ajaxBaseUrl}/aliyunConfig/getOss`,
    method: "get",
    data: {
      env
    }
  }).then(res => res.data);
}

